@@ -67,7 +67,7 @@ Changing a class method, by example
 
 =cut
 
-$Sub::Mage::VERSION = '0.012';
+$Sub::Mage::VERSION = '0.014';
 $Sub::Mage::Subs = {};
 $Sub::Mage::Imports = [];
 $Sub::Mage::Classes = [];
@@ -119,6 +119,7 @@ sub import {
                 duplicate
                 exports
                 have
+                around
             /,
         );
     }
@@ -161,7 +162,7 @@ sub _setup_class {
     my $class = shift;
 
     *{ "$class\::new" } = sub { return bless { }, $class };
-    _import_def ($class, qw/augment accessor/);
+    _import_def ($class, qw/augment accessor chainable/);
 }
 
 sub _import_def {
@@ -272,22 +273,75 @@ sub before {
         $pkg = caller;
     }
 
+    if (ref($name) eq 'ARRAY') {
+        for my $subname (@$name) {
+            $full = "$pkg\:\:$subname";
+            my $alter_sub;
+            my $new_code;
+            my $old_code;
+            die "Could not find $subname in the hierarchy for $pkg\n"
+                if ! $pkg->can($subname);
+
+            $old_code = \&{$full};
+            *$subname = sub {
+                $sub->(@_);
+                $old_code->(@_);
+            };
+
+            _add_to_subs($full);
+            *{$full} = \*$subname;
+            _debug("Added before hook modifier to $subname");
+        }
+    }
+    else {
+        $full = "$pkg\:\:$name";
+        my $alter_sub;
+        my $new_code;
+        my $old_code;
+        die "Could not find $name in the hierarchy for $pkg\n"
+            if ! $pkg->can($name);
+
+        $old_code = \&{$full};
+        *$name = sub {
+            $sub->(@_);
+            $old_code->(@_);
+        };
+
+        _add_to_subs($full);
+        *{$full} = \*$name;
+        _debug("Added before hook modifier to $name");
+    }
+}
+
+sub around {
+    my ($pkg, $name, $sub) = @_;
+
+    if (scalar @_ > 2) {
+        ($pkg, $name, $sub) = @_;
+    }
+    else {
+        ($name, $sub) = ($pkg, $name);
+        $pkg = caller;
+    }
+
     $full = "$pkg\:\:$name";
-    my $alter_sub;
-    my $new_code;
-    my $old_code;
     die "Could not find $name in the hierarchy for $pkg\n"
         if ! $pkg->can($name);
 
-    $old_code = \&{$full};
+    my $old_code = \&{$full};
     *$name = sub {
-        $sub->(@_);
-        $old_code->(@_);
+        $sub->($old_code, @_);
     };
-
+     
     _add_to_subs($full);
-    *{$full} = \*$name;
-    _debug("Added before hook modifier to $name");
+    *{$full} = \*$name;  
+}
+
+sub getscope {
+    my ($self) = @_;
+
+    if (defined $self) { return ref($self); }
+    else { return scalar caller(1); }
 }
 
 sub conjur {
@@ -416,6 +470,36 @@ sub accessor {
     };
 }
 
+sub chainable {
+    my ($method, %args) = @_;
+    my $pkg = getscope();
+
+    if (! $pkg->can($method)) {
+        warn "Cannot chain subroutine that doesn't exist";
+        return ;
+    }
+    
+    foreach my $var (keys %args) {
+        if ($var eq 'class') {
+            
+            if ($args{$var} eq $pkg) {
+                $pkg->after( $method => sub {
+                    my $self = shift;
+                    
+                    return $self;
+                });
+            }
+            else {
+                $pkg->after( $method => sub {
+                    my $self = shift;
+
+                    return bless $self, $args{$var};
+                });
+            }
+        }
+    }
+} 
+
 sub _debug_on {
     $Sub::Mage::Debug = 1;
     _debug("Sub::Mage debugging ON");
@@ -529,6 +613,45 @@ Very similar to C<after>, but calls the before subroutine, yes that's right, bef
 
     bye(); # prints Good Bye!
 
+Fancy calling C<before> on multiple subroutines? Sure. Just add them to an array.
+
+    sub like {
+        my ($self, $what) = @_;
+        
+        print "I like $what\n";
+    }
+    
+    sub dislike {
+        my ($self, $what) = @_;
+        
+        print "I dislike $what\n";
+    }
+
+    before [qw( like dislike )] => sub {
+        my ($self, $name) = @_;
+
+        print "I'm going to like or dislike $name\n";
+    };
+
+=head2 around
+
+Around gives the user a bit more control over the subroutine. When you create an around method the first argument will be the old method, the second is C<$self> and the third is any arguments passed to the original subroutine. In a away this allows you to control the flow of the entire subroutine.
+
+    sub greet {
+        my ($self, $name) = @_;
+
+        print "Hello, $name!\n";
+    }
+
+    # only call greet if any arguments were passed to Class->greet()
+    around 'greet' => sub {
+        my $method = shift;
+        my $self = shift;
+
+        $self->$method(@_)
+            if @_;
+    };
+
 =head2 conjur
 
 "Conjurs" a subroutine into the current script or a class. By conjur I just mean create. It will not allow you to override a subroutine 
@@ -693,6 +816,42 @@ accessor it adds the subroutine for you with the specified default value. The pa
     
     print "Seeya, " . $foo->name; # prints Seeya, Foo
 
+=head2 chainable
+
+Another C<:Class> only method is C<chainable>. It doesn't really do anything you can't do yourself, but I find it helps to keep a visual of your chains at the top of your code so you can see in plain sight 
+where they are leading you. Let's look at an example.
+
+    # test.pl
+
+    use Greeter;
+    
+    my $foo = Greeter->new;
+    print "Hello, " . $foo->greet('World')->hello;
+
+    # Greeter.pm
+    package Greeter;
+
+    use Greet::Class;
+    use Sub::Mage qw/:Class/;
+
+    chainable 'greet' => ( class => 'Greet::Class' );
+
+    sub greet {
+        my ($self, $name) = @_;
+        $self->{_name} = $name;
+    }
+
+    # Greet/Class.pm
+    package Greet;
+    
+    sub hello {
+        my $self = shift;
+
+        return $self->{_name};
+    }
+
+Out of all that we get 'Hello, World'. It may seem pointless, but I like a clean presentation plus keeping track of where methods point to, and I find using C<chainable> helps me with this.
+
 =head1 AUTHOR
 
 Brad Haywood <brad@geeksware.net>
